set more off
clear

use "$dropbox/data/input/cps_00017.dta"

keep if year == 2015



// REPLACE MISSING VALUES

replace fedtaxac = . if fedtaxac == 999999
replace statax = . if statax == 999999
replace incwelfr = . if incwelfr == 99999
replace incwage = . if incwage == 9999999
replace fica = . if fica == 99999
replace eitcred = . if eitcred == 9999
replace incss = . if incss == 99999
replace incretir = . if incretir == 999999
replace incssi = . if incssi == 99999
replace incunemp = . if (incunemp == 99999 | incunemp == 99997)
replace incdisab = . if incdisab == 999999



// RESTRICT TO EITC POPULATION FOR MSWW ANALYSIS

keep if inlist(ftype, "primary family":FTYPE, "nonfamily householder":FTYPE)
keep if relate == "head/householder":RELATE

keep if ftotval > 0
keep if inrange(age,25,55)
keep if nchild==2



// IMPORT MARGINAL TAX RATES FROM NBER TAXSIM

gen mstat = .
replace mstat = 1 if inlist(filestat,3,4)
replace mstat = 2 if inlist(filestat,1,2)


gen agex = 0

gen depx = nchild


gen pwages = incwage
gen swage = ftotval - pwages if mstat == 2

gen depchild = depx


taxsim9, replace
save "$dropbox/data/intermediate/taxsim_results.dat", replace

gen mtr_taxsim = (frate + srate + ficar)*0.01



// COMPUTE MARKET INCOME AND NET TAXES AND BENEFITS

gen mktinc = ftotval - incwelfr - incunemp - incss - incssi
drop if mktinc < 0
gen benef =  heatval + houssub + stampval + incwelfr

gen nettax = fedtaxac + stataxac + fica - benef



// CONVERT NOMINAL TO REAL 2010 dollars

gen cpi10 = cpi99/.777
foreach var of varlist ftotval mktinc nettax benef {
	replace `var' = `var'*cpi10
}


// COMPUTE IMPLICIT MTR FROM PHASEOUT OF BENEFITS

lpoly benef mktinc, bw(5000) nograph at(mktinc) gen(benefpoly)

gen mktinc_incr = mktinc + 1
lpoly benef mktinc, bw(5000) nograph at(mktinc_incr) gen(benefpoly_incr)

gen mtr_phaseout = -(benefpoly_incr - benefpoly)
gen mtr = mtr_taxsim + mtr_phaseout


// COLLAPSE DATA INTO INCOME BINS

local stepsize 2500

gen inc = mktinc
egen incbin = cut(mktinc), at(0(`stepsize')500000)

collapse (count) numobs=ftotval (mean) inc=mktinc (mean) mtr* *rate ficar nettax [iw=hwtsupp], by(incbin)
drop if missing(incbin)

gen dens = numobs/`stepsize'



// GENERATE SMOOTH INCOME DENSITY USING POLYNOMIAL REGRESSION

gen incsc = inc/1000
gen incsc2 = incsc^2/2
gen incsc3 = incsc^3/(2*3)
gen incsc4 = incsc^4/(2*3*4)
gen incsc5 = incsc^5/(2*3*4*5)

reg dens incsc* if inc < 300000
predict f2 if inc < 300000
gen df2 = _b[ incsc] + _b[ incsc2]* incsc + _b[incsc3]*incsc2 + _b[incsc4]*incsc3 + _b[incsc5]*incsc4
gen alpha = -(1 + df2/f2*incsc)


// SET INTENSIVE ELASTICITIES


local ecompLo 0.31
local ecompMid 0.14
local ecompHi 0.3
local ecompLoBd 10000 // center of transition between ecompLo and ecompMid
local ecompHiBd 30000 // center of transition between ecompMid and ecompHi
local ecompTransBandLo 20000 // width of transition band starting at ecompLoBd
local ecompTransBandHi 30000 // width of transition band starting at ecompHiBd

gen ecomp = `ecompLo'
replace ecomp = `ecompLo' + (`ecompMid' - `ecompLo')*(inc - `ecompLoBd')/`ecompTransBandLo' if inrange(inc,`ecompLoBd',`ecompLoBd'+`ecompTransBandLo')
replace ecomp = `ecompMid' if inc > `ecompLoBd'+`ecompTransBandLo'
replace ecomp = `ecompMid' + (`ecompHi' - `ecompMid')*(inc - `ecompHiBd')/`ecompTransBandHi' if inrange(inc,`ecompHiBd',`ecompHiBd'+`ecompTransBandHi')
replace ecomp = `ecompHi' if inc > `ecompHiBd'+`ecompTransBandHi'



// SET EXTENSIVE MARGIN ELASTICITY, SMOOTHING ACROSS 20K RANGE

local eBottom 0.09
local eLowBd 30000
local eUpBd 60000

gen epartic = `eBottom'
replace epartic = `eBottom'*(1 - (inc-`eLowBd')/(`eUpBd'-`eLowBd')) if inrange(inc,`eLowBd',`eUpBd')
replace epartic = 0 if inc >= `eUpBd'




// CALIBRATE BETA ACROSS INCOMES

gen beta = 0.40017 + (0.0072986 * inc/1000)
replace beta = 1 if beta > 1

gen wedge = 1 - beta



// GENERATE "FISCAL EXTERNALITY" OF LOCALIZED $1 TAX CREDIT

gen fe = -ecomp*(mtr/(1-mtr))*alpha - epartic*((nettax-nettax[1])/(inc-nettax+nettax[1]))

gen taxrat = mtr/(1-mtr)
gen emptax = (nettax-nettax[1])/(inc-nettax+nettax[1])
gen term1 = -ecomp*(mtr/(1-mtr))*alpha
gen term2 = - epartic*((nettax-nettax[1])/(inc-nettax+nettax[1]))


// GENERATE "WELFARE INTERNALITY" OF LOCALIZED $1 TAX CREDIT

gen wi = ecomp*wedge*alpha + epartic*wedge // includes participation effect


// COMPUTE TRADITIONAL (NO PRESENT BIAS) INVERSE OPTIMUM MSWWS 

egen tot = sum(numobs)
gen cumobs = sum(numobs)
gen quantile = cumobs/tot
gen pmf = numobs/tot
gen fe_wtd = fe*pmf
egen fe_exp = sum(fe_wtd)

gen msww = (1+fe)/(1+fe_exp)



// COMPUTE MSWWS ACCOUNTING FOR PRESENT BIAS

gen muc = (1+fe)/(1+wi)

gen muc_wtd = muc*pmf
egen muc_exp = sum(muc_wtd)

gen mswwalt = muc/muc_exp


// PLOT MSWW SCHEDULES

gen fwtsupp = round(numobs/100)
lpoly msww inc [fw=fwtsupp], bw(10000) nograph at(inc) gen(mswwsm)
lpoly mswwalt inc [fw=fwtsupp], bw(10000) nograph at(inc) gen(mswwaltsm)
lpoly mtr inc [fw=fwtsupp], bw(10000) nograph at(inc) gen(mtrsm)

twoway (scatter msww incsc, msymbol(Oh) mcolor(gs12)) (scatter mswwalt incsc, msymbol(Sh)  mcolor(gs12)) ///
	(line mswwsm incsc, lcolor(black) lwidth(0.5)) (line mswwaltsm incsc, lcolor(black) lpattern(dash)  lwidth(0.5)) if incsc < 150, ///
	graphregion(color(white)) ///
	legend(order(3 "Conventional MSWWs" 4 "MSWWs with present bias")) ///
	xtitle("Household market income ($1000s)") ytitle("Welfare weight") xlabel(0(25)150)
	
graph export "$github/analysis/output/figures/mswws_by_income.pdf", replace
